import os
import pandas as pd

from dtk.utils.analyzers import default_select_fn, default_group_fn, default_filter_fn, \
    default_plot_fn, TimeseriesAnalyzer


class DockerTimeseriesAnalyzer(TimeseriesAnalyzer):
    data_group_names = ['group', 'sim_id', 'channel']
    ordered_levels = ['channel', 'group', 'sim_id']
    output_file = 'timeseries.csv'

    def __init__(self, filename=os.path.join('output', 'InsetChart.json'), filter_function=default_filter_fn,
                 select_function=default_select_fn, group_function=default_group_fn, plot_function=default_plot_fn,
                 channels=('Statistical Population',
                           'Rainfall', 'Adult Vectors',
                           'Daily EIR', 'Infected',
                           'Air Temperature'), saveOutput=False):
        super().__init__(filename=filename, filter_function=filter_function, select_function=select_function,
                         group_function=group_function, plot_function=plot_function, channels=channels, saveOutput=saveOutput)

    def finalize(self, all_data):
        selected = []
        for sim, data in all_data.items():
            # Enrich the data with info
            data.group = self.group_function(sim.id, sim.tags)
            data.sim_id = sim.id
            selected.append(data)

        if len(selected) == 0:
            print("No data have been returned... Exiting...")
            return

        # Combining selected data...
        combined = pd.concat(selected, axis=1,
                             keys=[(d.group, d.sim_id) for d in selected],
                             names=self.data_group_names)

        # Re-ordering multi-index levels...
        data = combined.reorder_levels(self.ordered_levels, axis=1).sort_index(axis=1)

        if self.saveOutput:
            data.to_csv(self.output_file)

        def plot_fn(channel, ax):
            self.plot_function(data[channel].dropna(), ax)

        channels = data.columns.levels[0]
        self.plot_by_channel(channels, plot_fn)

        import matplotlib
        matplotlib.use("Agg")
        import matplotlib.pyplot as plt
        plt.legend()
        # plt.show()
        # plt.savefig(os.path.join(os.getcwd(), 'TimeSeries.pdf'), format='PDF')
        plt.savefig(os.path.join(os.getcwd(), '{}_plot.png'.format(self.__class__.__name__)), format='PNG')
        # plt.close()

    # Subplots by channel
    @staticmethod
    def plot_by_channel(channels, plot_fn):
        import numpy as np
        import matplotlib
        matplotlib.use("Agg")
        import matplotlib.pyplot as plt
        ncol = int(1 + len(channels) / 4)
        nrow = int(np.ceil(float(len(channels)) / ncol))

        fig, axs = plt.subplots(figsize=(max(6, min(8, 4 * ncol)), min(6, 3 * nrow)),
                                nrows=nrow, ncols=ncol,
                                sharex=True)

        flat_axes = [axs] if ncol * nrow == 1 else axs.flat
        for (channel, ax) in zip(channels, flat_axes):
            ax.set_title(channel)
            plot_fn(channel, ax)